#! /usr/local/bin/perl
# ESA (C) 2000-2018 
#
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#
# $Id: Makefile.PL,v 1.4 2013/04/01 11:42:20 eojero Exp $

use Env;
use ExtUtils::MakeMaker;
use FileHandle;

# See lib/ExtUtils/MakeMaker.pm for details of how to influence
# the contents of the Makefile that is written.

WriteMakefile(
	      'NAME'	=> 'pedal',
	      'CONFIGURE' => \&initialize,
##	      'VERSION_FROM' => 'DAL/DAL.pm',
	      );

sub initialize {
    my $self = {};

    die "Have you defined SAS_DIR?\n" if not defined $SAS_DIR;

    my $fh = new FileHandle;

    print "-----------------------------------\n";
    print "XMM Perl Enhanced Data Access Layer\n";
    print "-----------------------------------\n";
    print "Using SAS_DIR=$SAS_DIR\n";

    if($fh->open("$SAS_DIR/RELEASE", "r")) {
	my $sasver = <$fh>;
	chomp $sasver;
	print "Found SAS version: $sasver\n";
    }

    $fh->open("../VERSION", "r") or die "../VERSION: $!";
    my $ver = <$fh>;
    chomp $ver;
    $self->{'VERSION'} = $ver;

    print "\nThis is the pedal version $ver\n\n";

    $fh->open("../DEPEND", "r") or die "../DEPEND: $!";
    my $need_this_dal;
    while(<$fh>) {
	chomp;
	/\s*dal\s*(\d+(?:\.\d+)?)/ and do {
	    $need_this_dal = $1;
	    print "Need dal version $need_this_dal or better... ";
	    last;
	};
    }
	
    $fh->open("$SAS_DIR/packages/dal/VERSION", "r") or
	die "$SAS_DIR/packages/dal/VERSION: $!";
    my $have_this_dal = <$fh>;
    chomp $have_this_dal;
    print "found $have_this_dal. ";
    $have_this_dal =~ /(\d+\.\d+)/;
    $have_this_dal = $1;

    ($have_this_dal >= $need_this_dal) ?
	print "Ok.\n\n" : print "Oops, the compilation may fail.\n\n";
    
    undef $fh;

    $self->{'dist'} = {COMPRESS => 'gzip',
		       SUFFIX => 'gz',
		       };
		       

    return $self;
}
