#! /usr/local/bin/perl
# ESA (C) 2000-2018 
#
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
BEGIN { $| = 1; print "1..1\n"; }
use DAL;

$set = "abc.ds";

my $ds = DAL::dataset->new(name => $set,
			   mode => $DAL::CREATE);

my $tb = $ds->addTable(name => 'TABLE',
		       nrows => 64);

die("Cannot find block.\n") unless $ds->hasBlock(name => 'TABLE');

foreach( $DAL::BOOL, $DAL::INT8, $DAL::INT16, $DAL::REAL32, $DAL::REAL64, $DAL::STRING) {

    print "Type = $_\n";
    $tb->addAttribute(name => "F_$_",
		      datatype => $_,
		      value => 1,
		      label => "Comment",
		      units => "Units"
		      );
    
    die("Cannot find attribute F_$_.\n") if not $tb->hasAttribute(name => "F_$_");
    $tb->deleteAttribute(name => "F_$_");
    die("Can find attribute F_$_.\n") if $tb->hasAttribute(name => "F_$_");
    
    
    my $co = $tb->addColumn(name => "C_$_",
			    units => 'unit',
			    label => 'Comment',
			    datatype => $_);
    
    print "Column has dimensions ",$co->dimensions(),"\n";
    
    die if $co->isVariableLength();
    
    my $v = $co->data();
    
    $co->addAttribute(name => 'tlmin', datatype => $DAL::INT32);
    die("tlmin") unless $co->hasAttribute(name => 'tlmin');
    my $a = $co->attribute(name => 'tlmin');
    $a->set(123);
    print "Attribute = ", $a->asInt32(),"\n";
    die unless $a->asInt32() == 123;
    
    # Here must set only 0 or 1, because the type of the column can be
    # boolean or short integer, in which case we risk
    # segfaulting. This is really a limitation of the way I
    # implemented $v->set().
    for($i=0;$i<$tb->rows();++$i){ $v->set($i, $i % 2) }
    
    die("Cannot find column C_$_.\n") unless $tb->hasColumn(name => "C_$_");
    
    $co->release();
    $tb->deleteColumn(name => "C_$_");
    die("Can find column C_$_.\n") if $tb->hasColumn(name => "C_$_");
}

$ds->release;

##unlink $set;

print "ok 1\n";


